/**
 *  @file      TransactionManager.java
 *  @author    Stefan Reichert
 *
 *  Copyright (c)
 *  BTC Embedded Systems AG
 *  Oldenburg, Germany
 *
 *  --------------------------------
 *  $$Author: ${user} $$
 *  $$Date: ${date} ${time} $$
 *  $$Rev: revision number $$
 *  --------------------------------
 */
package com.btc.ep.base.transactions;

//---------------------------------------------------------------------------
//Implementation
//---------------------------------------------------------------------------

/**
 * This class represents the interface for a transaction manager implementation. It is expected to be used for methods
 * that represent a unit of work only.
 */
public interface TransactionManager {

    /**
     * Initializes the transaction by creating an appropriate {@link TransactionContext} for the
     * current thread. The {@link TransactionContext} should be available via the OSGi service bus so it can easily be
     * used by other parts of the application requiring access to the transactional objects.
     *
     * @param jointPointId The id of the current jointpoint the transaction applies to
     * @param isRollbackOnly flag whether the transaction is markes as read only. If <code>true</code>, the transaction
     *            is expected to be automatically rolled back on commit.
     * @return the context of the transaction providing access to transactional data
     * 
     * @throws IllegalStateException in case required collaborators are missing and no transaction handling can be
     *             established
     */
    TransactionContext init(String jointPointId, boolean isRollbackOnly) throws IllegalStateException;

    /**
     * Committing all changes made to transactional objects provided by the {@link TransactionContext}.
     *
     * @param jointPointId The id of the current jointpoint the transaction applies to
     */
    void commit(String jointPointId);

    /**
     * Rolling back all changes made to transactional objects provided by the {@link TransactionContext}.
     *
     * @param jointPointId The id of the current jointpoint the transaction applies to
     * @param exception The cause for the rollback
     */
    void rollback(String jointPointId, RuntimeException exception);

    /**
     * Cleaning, closing and removing all transactional objects provided by the {@link TransactionContext}.
     *
     * @param jointPointId The id of the current jointpoint the transaction applies to
     */
    void close(String jointPointId);

}